import { Modal, App, Radio, Popover } from 'antd';
import React, { useEffect, useState } from 'react';
import { useSetState } from 'ahooks';
import { useTranslation } from 'react-i18next';
import { TFunction } from 'i18next';

import * as article from '@/service/api/article.ts';
import { regenerate } from '@/service/api/video.ts';
import { push2video } from '@/service/api/article.ts';
import { showErrorToast, showToast } from '@/components/message.ts';
import ArticleGroup, { HotNewsData } from '@/components/article/group.tsx';
import type { HookAPI as ModalHookAPI } from 'antd/es/modal/useModal';
import { IconWarningCircle } from '@/components/icons';

import Bg1 from './assets/bg1.jpg'
import Bg2 from './assets/bg2.jpg'

type Props = {
	id?: number;
	type: 'news' | 'video';
	onClose?: (saved?: boolean) => void;
	onRefresh?: () => void
}

const DEFAULT_STATE = {
	loading: false,
	open: false,
	msgTitle: '',
	msgGroup: '',
	error: ''
};

function pushBlocksToGroup(blocks: BlockContent[], groups: BlockContent[][]) {
	const lastGroup = groups[groups.length - 1];
	if (lastGroup && lastGroup.filter(s => s.type == 'text').length == 0) {
		// 如果上一个group中没有文本则直接合并
		lastGroup.push(...blocks);
	} else {
		groups.push(blocks);
	}
}

function rebuildGroups(groups: BlockContent[][]) {
	const _groups: BlockContent[][] = [];
	if (!groups || groups.length == 0) return _groups;
	groups.forEach((blocks, index) => {
		if (!blocks) return;
		blocks = blocks.filter(s => !!s).sort((a, b) => {
			if (a.type == 'text' && b.type == 'text') return 1;
			return a.type == 'text' ? -1 : 1;
		});
		if (blocks.length == 1) {
			if (index == 0) _groups.push(blocks);
			else pushBlocksToGroup(blocks, _groups);
		} else {
			if (index == 0) {
				_groups.push([blocks[0]]);
				_groups.push(blocks.slice(1));
			} else {
				pushBlocksToGroup(blocks, _groups);
			}
		}
	});
	if (_groups.length < 2) {
		Array(2 - _groups.length).fill([{ type: 'text', content: '' }]).forEach((it) => {
			_groups.push(it);
		});
	}
	// console.log('rebuildGroups', _groups)
	return _groups;


}

function groupHasImageAndText(blocks: BlockContent[]) {
	return blocks.some(s => s.type == 'image' && s.content.trim().length > 0) && blocks.some(s => s.type == 'text' && s.content.trim().length > 0);
}

// 验证分组数据是否合法
function checkGroupsValid(_groups: BlockContent[][]) {
	const groups = _groups.filter((_, index) => {
		if (index == 0) return true;
		return _.length > 1 || (_.length == 1 && _[0].content.trim().length > 0);
	});
	if (groups.length == 1) return true;
	for (let index = 1; index < groups.length; index++) {
		if (!groupHasImageAndText(groups[index])) return false;
	}
	return true;
}

function checkHotNewsValid(hotNews: HotNewsData, modal: ModalHookAPI, t: TFunction<'translation', undefined>) {
	return new Promise<boolean>((resolve) => {

		// 验证热点新闻数据是否正确
		if (hotNews.mode == 'manual' && hotNews.list.filter(s => s.trim().length > 0).length < 3) {
			modal.warning({
				wrapClassName: 'root-modal-confirm',
				icon: <span className="anticon anticon-exclamation-circle"><IconWarningCircle /></span>,
				title: t('modal.hot_news.empty_notice_title'),
				content: <span dangerouslySetInnerHTML={{ __html: t('modal.hot_news.empty_notice_message') }}></span>,
				centered: true,
				onOk: () => {
					resolve(false);
				},
				onCancel: () => {
					resolve(false);
				}
			});
			return;
		}
		resolve(true);
	});
}

export default function ArticleEditModal(props: Props) {
	const { t, i18n } = useTranslation();
	const { modal } = App.useApp();
	const [groups, setGroups] = useState<BlockContent[][]>([]);
	const [title, setTitle] = useState('');
	const [tag, setTag] = useState('');
	const [articleTemplateInfo, setArticleTemplateInfo] = useState<ArticleTemplateInfo>({
		select:'',
		options:[]
	});
	const [hotNews, setHotNews] = useState<HotNewsData>({
		list: ['', '', ''],
		mode: 'auto'
	});
	const [state, setState] = useSetState({
		...DEFAULT_STATE,
		generating: false,
		pushed: false
	});

	// 保存数据
	const handleSave = async () => {
		setState({ error: '' });
		if (!title) {
			// setState({msgTitle: '请输入标题内容'});
			return;
		}
		if(i18n.language == 'zh-CN' && tag.length > 4 ){
			// 获取图文设置不正确的数据
			setState({ msgGroup: t('news.edit.tag_length_error') });
			return;
		}
		if (groups.length == 0 || groups[0].length == 0 || !groups[0][0].content) {
			setState({ msgGroup: t('news.edit_empty_human_content') });
			return;
		}
		// 验证图文都存在时，文图是否匹配
		if (!checkGroupsValid(groups)) {
			// 获取图文设置不正确的数据
			setState({ msgGroup: t('news.edit_empty_group_content') });
			return;
		}
		const hotNewsValid = await checkHotNewsValid(hotNews, modal, t);
		if (!hotNewsValid) return;
		// if (groups.length == 0 || groups[0].length == 0 || !groups[0][0].content) {
		// 	// setState({msgGroup: '请输入正文文本内容'});
		// 	return;
		// }
		const save = props.type == 'news' ? article.save : regenerate;
		setState({ loading: true });
		save({
			title,
			metahuman_text: groups[0][0].content,
			content_group: groups.slice(1),
			hot_news: hotNews.mode == 'auto' ? [''] : hotNews.list,
			video_tag:tag,
			background: articleTemplateInfo.select,
			id: props.id && props.id > 0 ? props.id : undefined
		}).then(() => {
			props.onClose?.(true);
		}).catch(e => {
			setState({ error: e.message || t('news.edit_save_failed') });
		}).finally(() => {
			setState({ loading: false });
		});
	};
	const handlePush2Video = async () => {
		if (state.pushed) return;
		if (!title) {
			// setState({msgTitle: '请输入标题内容'});
			return;
		}
		if(i18n.language == 'zh-CN' && tag.length > 4 ){
			// 获取图文设置不正确的数据
			setState({ msgGroup: t('news.edit.tag_length_error') });
			return;
		}
		if (groups.length == 0 || groups[0].length == 0 || !groups[0][0].content) {
			setState({ msgGroup: t('news.edit_empty_human_content') });
			return;
		}
		// 验证图文都存在时，文图是否匹配
		if (!checkGroupsValid(groups)) {
			// 获取图文设置不正确的数据
			setState({ msgGroup: t('news.edit_empty_group_content') });
			return;
		}
		if (!props.id || state.generating) return;
		const hotNewsValid = await checkHotNewsValid(hotNews, modal, t);
		if (!hotNewsValid) return;
		setState({ generating: true });
		await article.save({
			title,
			metahuman_text: groups[0][0].content,
			content_group: groups.slice(1),
			hot_news: hotNews.mode == 'auto' ? [''] : hotNews.list,
			video_tag:tag,
			background: articleTemplateInfo.select,
			id: props.id
		});
		push2video([props.id]).then(() => {
			showToast(t('news.push_stream_success'), 'success');
			setState({ pushed: true });
			props.onClose?.(true);
			// props.onRefresh?.();
			// navigate('/create?state=push-success',{
			// 	state: 'push-success'
			// })
			// props.onSuccess?.()
		}).catch(showErrorToast).finally(() => {
			setState({ generating: false });
		});
	};
	useEffect(() => {
		setState({ ...DEFAULT_STATE });
		if (typeof (props.id) != 'undefined') {
			// 如果传入了id则获取数据
			if (props.id > 0) {
				article.getById(props.id).then(res => {
					if (res.hot_news) {
						const len = res.hot_news.length;
						const list = len >= 3 ? res.hot_news : res.hot_news.concat(Array(3 - len).fill(''));
						const mode = res.hot_news && res.hot_news.filter(s => s.length > 0).length == 3 ? 'manual' : 'auto';
						setHotNews({
							list,
							mode
						});
					}
					setGroups(rebuildGroups([[{ content: res.metahuman_text, type: 'text' }], ...res.content_group]));
					setTitle(res.title);
					setTag(res.video_tag)
					setArticleTemplateInfo(res.template_info)
				});
			} else {
				// 新增
				setGroups([]);
				setTitle('');
			}
		}
	}, [props.id]);

	return (<Modal
		title={null}
		centered={true}
		rootClassName={'article-edit-modal'}
		open={props.id != undefined && props.id >= 0}
		maskClosable={false}
		keyboard={false}
		width={'1200px'}
		footer={null}
		closeIcon={null}
		onCancel={() => props.onClose?.()}
		okButtonProps={{ loading: state.loading }}
		onOk={handleSave}
		okText={props.type == 'news' ? t('confirm_text') : t('news.edit_generate_video_again')}
	>
		<div className="mt-5 px-6 pt-10">
			<div className="flex items-center pb-3 article-title">
				<span className="mr-2 text-lg">{t('news.title')}</span>
				<input className={'input-box text-lg flex-1 py-2'} value={title} onChange={e => {
					setTitle(e.target.value);
					setState({ msgTitle: e.target.value ? '' : t('news.edit_notice_enter_article_title1') });
				}} placeholder={t('news.edit_notice_enter_article_title')} />
			</div>
			<div className="text-red-500 mt-2">{state.msgTitle}</div>
		</div>
		<div className="article-body">
			<div className="box text-base">
				<ArticleGroup
					errorMessage={state.msgGroup}
					editable
					groups={groups}
					hotNews={hotNews}
					onChange={(list, hotNews) => {
						setHotNews(hotNews);
						setGroups(() => list);
						setState({ msgGroup: (list.length == 0 || list[0].length == 0 || !list[0][0].content) ? t('news.edit_empty_human_content') : '' });
					}}
					leftPanelHeader={<div>
						<div className="row tag flex items-center mt-2">
							<span className="mr-2">{t('news.edit.tag')}</span>
							<input className={'input-box flex-1 py-1.5'} value={tag} onChange={e => {
								setTag(e.target.value);
							}} placeholder={t('news.edit.tag_placeholder')} />
						</div>
						<div className="row bg flex items-center my-3">
							<span className="mr-2">{t('news.edit.bg')}</span>
							<div className="bg-radio-container">
								<Radio.Group
									value={articleTemplateInfo.select}
									onChange={e=>{
									setArticleTemplateInfo(prev=>(
										{
											...prev,
											select: e.target.value
										}
									))
								}}>
									{articleTemplateInfo.options.map((opt,idx)=>(
										<Popover
											rootClassName="background-template-popover"
											key={idx} placement="bottomLeft" arrow={false}
											content={<img className="w-[150px] rounded" src={opt.background} />}
										>
											<Radio value={opt.template_id}>{t('news.edit.bg')}{idx + 1}</Radio></Popover>
									))}
								</Radio.Group>
							</div>
						</div>
					</div>}
				/>
				<div className="text-red-500 mt-2">{state.msgGroup}</div>
			</div>
			{state.error && <div className="text-red-500 mt-2">{state.error}</div>}
		</div>
		<div className="modal-control-footer flex justify-end">
			<div className="flex gap-10 ">
				{props.type == 'news' && props.id ? <button
					className="text-gray-400 hover:text-gray-800"
					onClick={handlePush2Video}
				>
					{t('news.edit_generate_video')}{state.pushed ? `${i18n.language == 'zh-CN' ? '中' : ''}...` : (state.generating ? `${i18n.language == 'zh-CN' ? '推送中' : 'Pushing'}...` : '')}
				</button> : null}
				<button className="text-gray-400 hover:text-gray-800" onClick={() => props.onClose?.()}>{t('cancel')}</button>
				<button onClick={handleSave}
				        className="text-gray-800 hover:text-blue-500">{props.type == 'news' ? t('news.save_text') : t('news.edit_generate_again')}</button>
			</div>
		</div>
	</Modal>);
}