import React, { useEffect, useState } from 'react';
import TCPlayer from 'tcplayer.js';
import 'tcplayer.js/dist/tcplayer.min.css';

import { PlayerInstance } from '@/hooks/useCache.ts';
import videoPlayer from '@/components/video/VideoPlayer.ts';

type State = {
	playing: boolean
	muted: boolean
	end?: boolean
	error?: boolean
	fullscreen: boolean
	progress: number
	playedSeconds: number
	duration: number
}
type StateUpdate = Partial<State> | ((prev: State) => Partial<State>)

type Props = {
	url?: string; cover?: string; showControls?: boolean; className?: string;
	poster?: string;
	onChange?: (state: State) => void;
	onProgress?: (current: number, duration: number) => void;
	muted?: boolean;
	autoPlay?: boolean;
}
export type PlayerInstance = {
	play: (url: string, currentTime: number) => void;
	pause: () => void;
	getState: () => State;
}
export const Player = React.forwardRef<PlayerInstance, Props>((props, ref) => {
	const [prevUrl, setPrevUrl] = useState<string | undefined>();
	const [tcPlayer, setTcPlayer] = useState<TCPlayerInstance | null>(null);

	const [state, _setState] = useState<State>({
		playing: false,
		muted: false,
		// 是否全屏
		fullscreen: false,
		progress: 0,
		playedSeconds: 0,
		duration: 0
	});

	const setState = (data: StateUpdate) => {
		console.log('playstate change', data);
		_setState(prev => {
			const _state = typeof (data) === 'function' ? { ...prev, ...data(prev) } : { ...prev, ...data };
			props.onChange?.(_state);
			return _state;
		});
	};

	useEffect(() => {
		if (props.url && tcPlayer) {
			tcPlayer.src(props.url);
		}
	}, [props.url, tcPlayer]);

	useEffect(() => {
		const playerVideo = videoPlayer.newInstance().getVideo();
		document.querySelector('.video-player-container-inner')!.appendChild(playerVideo);

		const flvPlayer = TCPlayer(playerVideo, {
				//sources: [{src: props.url}],
				controls: props.showControls,
				// muted:props.muted,
				poster: props.poster,
				autoplay: typeof (props.autoPlay) != 'undefined' ? props.autoPlay : true,
				licenseUrl: 'https://license.vod2.myqcloud.com/license/v2/1328581896_1/v_cube.license'
			}
		);

		flvPlayer.on('pause', () => {
			setState({ playing: false, end: false, error: false });
		});
		flvPlayer.on('playing', () => {
			setState({ playing: true, end: false, error: false });
		});
		flvPlayer.on('ended', () => {
			setState({ end: true, playing: false, error: false });
		});
		flvPlayer.on('timeupdate', () => {
			props.onProgress?.(flvPlayer.currentTime(), flvPlayer.duration());
		});
		flvPlayer.on('error', () => {
			setState({ end: false, playing: false, error: true });
		});
		setTcPlayer(() => flvPlayer);
		return () => {
			tcPlayer?.pause();
			console.log('destroy video');
			tcPlayer?.dispose();
		};
	}, []);

	React.useImperativeHandle(ref, () => {
		return {
			pause() {
				if (!tcPlayer) return;
				tcPlayer.pause();
			},
			play: (url, currentTime = 0) => {
				console.log('play', url, currentTime);
				if (!tcPlayer) return;
				const player = tcPlayer;
				if (prevUrl == url) {
					player.currentTime(0);
				} else {
					player.src(url);
				}
				player.play();
				setPrevUrl(url);
				if (currentTime > 0) {
					player.currentTime(currentTime);
				}
			},
			getState: () => state
		};
	});

	return <div className={`video-player relative ${props.className} video-player-container-inner`}>
	</div>;
});