import {Checkbox, Empty, Space} from "antd";
import React, {useEffect, useRef, useState} from "react";
import {DndContext} from "@dnd-kit/core";
import {arrayMove, SortableContext} from "@dnd-kit/sortable";
import {useSetState} from "ahooks";
import {useLocation, useNavigate} from "react-router-dom";
import {useTranslation} from "react-i18next";

import {VideoListItem} from "@/components/video/video-list-item.tsx";
import ArticleEditModal from "@/components/article/edit-modal.tsx";
import {deleteFromList, getList, modifyOrder, regenerateById, VideoStatus} from "@/service/api/video.ts";
import {formatDuration} from "@/util/strings.ts";
import ButtonBatch from "@/components/button-batch.tsx";
import {showErrorToast, showToast} from "@/components/message.ts";
import {Mp4Player as Player, PlayerInstance} from "@/components/video/Mp4Player.tsx";
import ButtonPush2Room from "@/pages/video/components/button-push2room.tsx";
import ButtonToTop from "@/components/scoller/button-to-top.tsx";
import InfiniteScroller, {InfiniteScrollerRef} from "@/components/scoller/infinite-scroller.tsx";
import {IconDelete} from "@/components/icons";

export default function VideoIndex() {
	const {t} = useTranslation()
	const [editId, setEditId] = useState(-1)
	const loc = useLocation()
	const navigate = useNavigate()
	const [videoData, setVideoData] = useState<VideoInfo[]>([])
	const player = useRef<PlayerInstance | null>(null)
	const scrollerRef = useRef<InfiniteScrollerRef | null>(null)
	const [state, setState] = useSetState({
		checkedAll: false,
		playingId: -1,
		videoPlaying: false,
		showToTop: false,
		showStatePos: false,
		playState: {
			current: -1,
			total: -1
		},
		loading:false,
		playVideoUrl: ''
	})
	const [checkedIdArray, setCheckedIdArray] = useState<Id[]>([])
	const [refreshTimer,setTimer] = useState(0)

	// 加载列表
	const loadList = (needReset = true) => {
		if(state.loading) return;
		if(refreshTimer) {
			clearTimeout(refreshTimer)
			setTimer(0)
		}
		setState({loading: true})
		getList().then((ret) => {
			const list = ret.list || []
			setVideoData(list)
			if (needReset) {
				setCheckedIdArray([])
				setState({checkedAll: false, playingId: -1})
			}
			// 判断是否有生成中的视频
			if (list.filter(s => s.status == VideoStatus.Generating).length > 0) {
				// 每5s重新获取一次最新数据
				setTimer(()=>setTimeout(() => loadList(false), 5000) as any);
			}
		}).catch(showErrorToast)
			.finally(()=>{
			setState({loading: false})
		})
		return ()=>{
			if(refreshTimer){

				clearTimeout(refreshTimer)
			}
			console.log('go out',refreshTimer)
		}
	}

	// 播放视频
	const playVideo = (video: VideoInfo) => {

		if (state.playingId == video.id) {
			player.current?.pause();
			setState({playingId: -1})
			return;
		}
		if (video.status == VideoStatus.Generating) return;
		// setState({playingIndex})
		// player.current?.play('https://staticplus.gachafun.com/ai-collect/composite_video/2024-12-17/1186196465916190720.flv', 30)
		//
		if (video.oss_video_mp4_url && video.status !== 1) {
			setState({playingId: video.id})
			player.current?.play(video.oss_video_mp4_url, 0)
		}
	}
	// 处理全选
	const handleAllCheckedChange = () => {
		setCheckedIdArray(state.checkedAll ? [] : videoData.filter(s=>s.status == VideoStatus.Generated).map(v => v.id))
		setState({
			checkedAll: !state.checkedAll
		})
	}
	const handleModifySort = (items: VideoInfo[]) => {

		modifyOrder(items.map(s => s.id)).then(() => {
			showToast(t('video.sort_modify_success'), 'success')
		}).catch(() => {
			loadList();
			showToast(t('video.sort_modify_failed'), 'warning')
		})

		return ()=>{
			try{
				Array.from(document.querySelectorAll('video')).forEach(v => v.pause())
			}catch (e){
				console.log(e)
			}
		}
	}
	//
	useEffect(loadList, [])
	// const totalDuration = useMemo(() => {
	// 	if (!videoData || videoData.length == 0) return 0;
	// 	const v = state.playingId == -1  ? null : videoData.find(s=>s.id == state.playingId)
	// 	if (!v) return 0
	// 	//const v = videoData[state.playingIndex] as VideoInfo;
	// 	return Math.ceil(v.duration / 1000)
	// 	// 计算总时长
	// 	//return videoData.reduce((sum, v) => sum + Math.ceil(v.duration / 1000), 0);
	// }, [videoData, state.playingId])

	useEffect(() => {
		if (loc.state == 'push-success' && !state.showStatePos && videoData.length && scrollerRef.current) {
			const generatingItem = document.querySelector(`.list-item-state-${VideoStatus.Generating}`)
			if (generatingItem) {
				generatingItem.scrollIntoView({behavior: 'smooth'})
				setState({showStatePos: true})
			}
		}
	}, [videoData, scrollerRef])
	const processDeleteVideo = async (ids: Id[],action ?: string) => {
		deleteFromList(ids).then(() => {
			showToast(t('delete_success'), 'success')
			if(action == 'rollback'){
				navigate('/edit',{
					state: {action: 'rollback',id: ids[0]},
				})
			}else{
				loadList()
			}
		}).catch(showErrorToast)
	}
	const processGenerateVideo = async (video: VideoInfo) => {
		regenerateById(video.article_id).then(() => {
			//showToast(t('delete_success'), 'success')
			loadList()
		}).catch(showErrorToast)
	}

	return (<div className="container py-5 page-live">
		<div className="h-[36px]"></div>
		<div className="flex">
			<div className="video-player-container mr-16 w-[360px] flex items-center">
				<div>
					<div className="text-center text-base text-gray-400">{t("generating.title")}</div>
					<div className="video-player flex items-center mt-2">
						<div className=" w-[360px] h-[636px] rounded overflow-hidden">
							{/*videoData[state.playingIndex]?.oss_video_url*/}
							<Player
								ref={player}
								url={state.playVideoUrl}
								onChange={(state) => {
									if (state.end || state.error) setState({playingId: -1})
								}}
								onPause={() => {
									setState({videoPlaying:false})
								}}
								onPlay={() => {
									setState({videoPlaying:true})
								}}
								onProgress={(current, duration) => {
									setState({
										playState: {
											current: current,
											total: duration
										}
									})
								}}
								className="w-[360px] h-[640px] bg-white"/>
						</div>
					</div>
					<div className="text-center text-sm mt-4 text-gray-400">{formatDuration(state.playState.current)} / {formatDuration(state.playState.total)}</div>
				</div>
			</div>
			<div className="video-list-container rounded mt-2 flex flex-col flex-1">
				<div className="live-control flex justify-between  h-[30px]">
					<div className="pl-[70px]"></div>
					<div className="flex items-center">
						<Space size={20}>
							<span>{t('select.total',{count:videoData.length || 0})}</span>
							<span className={'text-blue-500'}>{t('select.selected_some',{count:checkedIdArray.length})}</span>
						</Space>
						<button className="hover:text-blue-300 text-gray-400 ml-5"
						        onClick={handleAllCheckedChange}>
							<span className="text-sm mr-2">{t("select.select_all")}</span>
							{/*<CheckCircleFilled className={clsx({'text-blue-500': state.checkedAll})}/>*/}
						</button>
						<Checkbox checked={state.checkedAll} onChange={() => handleAllCheckedChange()}/>
					</div>
				</div>
				<div className={'video-list-sort-container flex-1 mt-1'}>
					<div className="list-header">
						<div className="list-row header-row">
							<div className="col number">No.</div>
							<div className="col cover">{t('video.title_thumb')}</div>
							<div className="col title">{t('video.title')}</div>
							<div className="col generated-time">{t('video.title_generated_time')}</div>
							<div className="col operation">{t('video.title_operation')}</div>
						</div>
					</div>
					<InfiniteScroller loading={state.loading} ref={scrollerRef} onScroll={top => setState({showToTop: top > 30})}>
						{
							videoData.length == 0 ? <div className="m-auto py-16"><Empty/></div> :
								<div className="sort-list-container flex-1">
									<DndContext onDragEnd={(e) => {
										const {active, over} = e;
										if (over && active.id !== over.id) {
											let oldIndex = -1, newIndex = -1;
											const originArr = [...videoData]
											console.log(originArr.map(s => s.id))
											setVideoData((items) => {
												oldIndex = items.findIndex(s => s.id == active.id);
												newIndex = items.findIndex(s => s.id == over.id);
												const newSorts = arrayMove(items, oldIndex, newIndex);
												handleModifySort(newSorts)
												return newSorts;
											});
										}
									}}>
										<SortableContext items={videoData}>
											{videoData.map((v, index) => (
												<VideoListItem
													video={v}
													index={index + 1}
													id={v.id}
													key={index}
													type={'create'}
													active={checkedIdArray.includes(v.id)}
													playing={state.playingId == v.id && state.videoPlaying}
													checked={checkedIdArray.includes(v.id)}
													className={`list-item-${index} mt-3 mb-2 list-item-state-${v.status} `}
													onCheckedChange={(checked) => {
														setCheckedIdArray(idArray => {
															const newArr = checked ? idArray.concat(v.id) : idArray.filter(id => id != v.id);
															setState({checkedAll: newArr.length == videoData.length})
															return newArr;
														})
													}}
													onItemClick={() => playVideo(v)}
													onRemove={(action) => processDeleteVideo([v.id],action)}
													onEdit={v.status == VideoStatus.Generated ? () => {
														setEditId(v.article_id)
													}:undefined}
													onRegenerate={v.status != VideoStatus.Generating && v.status != VideoStatus.Generated?()=>{
														processGenerateVideo(v).catch(console.log)
													}:undefined}
													downloadUrl={v.oss_video_mp4_url}
													hideCheckBox={v.status != VideoStatus.Generating && v.status != VideoStatus.Generated}
													editable={v.status != VideoStatus.Generating}
													sortable={v.status == VideoStatus.Generated}
												/>))}
										</SortableContext>
									</DndContext>
								</div>
						}
						<div className="h-[130px]"></div>
					</InfiniteScroller>
				</div>
			</div>
			<div className="page-action">
				<ButtonToTop visible={state.showToTop} onClick={() => scrollerRef.current?.scrollToPosition(0)}/>
				{
					checkedIdArray.length > 0 && <ButtonBatch
            onProcess={deleteFromList}
            selected={checkedIdArray}
            emptyMessage={t('video.delete_empty')}
            title={
							checkedIdArray.length == 1 ? t('video.delete_description', {count: checkedIdArray.length}) :
								t('video.delete_description_count', {count: checkedIdArray.length})
						}
            className='bg-gray-300 hover:bg-gray-400 text-white'
            confirmMessage={<span dangerouslySetInnerHTML={{
							__html: checkedIdArray.length == 1 ?
								t('video.delete_confirm', {count: checkedIdArray.length}) :
								t('video.delete_confirm_count', {count: checkedIdArray.length})
						}}></span>}
            onSuccess={() => {
							showToast(t('delete_success'), 'success')
							loadList()
						}}
            >
              <span className="text">{t('delete_batch')}</span>
              <IconDelete/>
            </ButtonBatch>
				}
				<ButtonPush2Room ids={checkedIdArray} list={videoData} onSuccess={loadList}/>
			</div>
		</div>
		<ArticleEditModal type={'video'} id={editId} onClose={(saved) =>{
			setEditId(-1)
			if(saved) {
				loadList()
			}
		}}/>
	</div>)
}