import {Outlet, useLocation, useNavigate} from "react-router-dom";
import {Divider, Dropdown, MenuProps} from "antd";
import React, {useEffect, useMemo} from "react";
import {useTranslation} from "react-i18next";

import AuthGuard from "@/routes/layout/auth-guard.tsx";
import {LogoText} from "@/components/icons/logo.tsx";

import {UserAvatar} from "@/components/icons/user-avatar.tsx";
import {DashboardNavigation} from "@/routes/layout/dashboard-navigation.tsx";

import useAuth from "@/hooks/useAuth.ts";
import {hidePhone} from "@/util/strings.ts";
import {defaultCache} from "@/hooks/useCache.ts";
import {IconOrderFill, IconRecycleFill} from "@/components/icons";
import LanguageSwitcher from "@/components/icons/language-switcher.tsx";


type LayoutProps = {
	children: React.ReactNode
}

const NavigationUserContainer = () => {
	const {t} = useTranslation()
	const {logout, user} = useAuth()
	const navigate = useNavigate()
	const handleLogout = () => {
		logout().then(() => navigate('/user'))
	}
	const items: MenuProps['items'] = [
		{
			key: 'order',
			label: <div className="nav-item" onClick={() => navigate('/order')}>
				<IconOrderFill/>
				<span className={"nav-text"}>{t('order.text')}</span>
			</div>,
		},
		{
			key: 'recycle',
			label: <div className="nav-item" onClick={() => navigate('/recycle')}>
				<IconRecycleFill/>
				<span className={"nav-text"}>{t('history.text')}</span>
			</div>,
		},
		// {
		// 	key: 'logout',
		// 	label: <div onClick={handleLogout}>退出</div>,
		// },
	];
	const UserButton = () => (<div
		className={`flex items-center rounded-3xl ${user ? 'bg-[#e3eeff]' : 'bg-primary-blue'} p-1 pr-2 cursor-pointer rounded`}>
		<UserAvatar className="user-avatar size-7"/>
		{user ? <span className={"username ml-2 text-sm"}>{hidePhone(user.nickname)}</span> : (
			<span className="text-sm mx-2 text-white">{t('login.title')}</span>
		)}
	</div>)
	return (<div className={"flex items-center justify-between gap-2 ml-10"}>
		{user ? <Dropdown
			rootClassName={'z-[999999] userinfo-drop-menu'}
			menu={{items}} placement="bottomRight"
			dropdownRender={(menu) => (
				<div>
					<div className="user-profile flex gap-4">
						<div className="avatar"><UserAvatar className="user-avatar"/></div>
						<div className="info">
							<div>{user?.nickname}</div>
							<div>ID: {user?.id}</div>
						</div>
					</div>
					<Divider style={{margin: 0}}/>
					<div className="menu-list-container">
						{menu}
					</div>
					<Divider style={{margin: 0}}/>
					<div className="logout">
						<div onClick={handleLogout}>{t('user.logout')}</div>
					</div>
				</div>

			)}
		>
			<div><UserButton/></div>
		</Dropdown> : <UserButton/>}
	</div>)
}
const ExtraNavItems = {
	'/order':'order.text',
	'/recycle':'history.text',
}
export const BaseLayout: React.FC<LayoutProps> = ({children}) => {
	const {pathname} = useLocation()
	const {t,i18n} = useTranslation()
	const extraNav = useMemo(()=>{
		if(!pathname || !ExtraNavItems[pathname]) return null
		return t(ExtraNavItems[pathname])
	},[pathname,i18n.language])
	return (<div className={'dashboard-layout min-h-screen'}>
		<div className="min-h-screen w-full">
			<div className="app-header">
				<div className="logo-container flex items-center">
					<LogoText style={{fontSize: 30}}/>
					{extraNav && <div className="extra-nav-name ml-2">
						<span className="nav-item active">{extraNav}</span>
					</div>}
				</div>
				<DashboardNavigation/>
				<div className="flex items-center">
					<LanguageSwitcher />
					<NavigationUserContainer/>
				</div>
			</div>
			<div className="app-content flex-1 box-sizing">
				<div className="content-container">
					{children}
				</div>
			</div>
		</div>
	</div>)
}


const DashboardLayout: React.FC<{ children?: React.ReactNode }> = ({children}) => {
	const loc = useLocation()
	const navigate = useNavigate()
	useEffect(() => {
		if (!defaultCache.firstLoadPath && loc.pathname == '/live') {
			defaultCache.firstLoadPath = loc.pathname;
			navigate('/')
		}
	}, [])
	return <AuthGuard>
		<div className="fixed">{defaultCache.firstLoadPath}</div>
		<BaseLayout>
			{children ? children : <Outlet/>}
		</BaseLayout>
	</AuthGuard>
}
export default DashboardLayout